// Copyright (C) 2024 Clyne Sullivan <clyne@bitgloo.com>
//
// Distributed under the GNU GPL v3 or later. You should have received a copy of
// the GNU General Public License along with this program.
// If not, see <https://www.gnu.org/licenses/>.

const cpu = @import("cpu.zig");
const gpio = @import("gpio.zig");
const interrupt = @import("interrupt.zig");
const timer = @import("timer.zig");

const rcc: *[39]u32 = @ptrFromInt(0x40021000);
const gpioa = gpio.gpioa;
const gpioc = gpio.gpioc;
const tick = timer.systick;

pub fn main() void {
    cpu.interrupt_disable();
    interrupt.initialize();
    interrupt.register(.SVCall, svcall);
    tick.initialize(1000);
    cpu.interrupt_enable();

    rcc[19] |= 5; // gpio a and c
    gpioa.set_mode(5, .output);
    gpioc.set_mode(13, .input);

    while (true) {
        asm volatile("svc 0");
        tick.delay(500);
    }
}

fn svcall() void {
    const state = gpioc.read(13);
    gpioa.write(5, state);
}

