// Copyright (C) 2024 Clyne Sullivan <clyne@bitgloo.com>
//
// Distributed under the GNU GPL v3 or later. You should have received a copy of
// the GNU General Public License along with this program.
// If not, see <https://www.gnu.org/licenses/>.

const interrupt = @import("interrupt.zig");

const driver_armcortex = packed struct {
    enable: u1,
    tickint: u1,
    unused: u30,

    rvr: u32,
    cvr: u32,
    calib: u32,

    pub fn initialize(self: *driver_armcortex, freq: u32) void {
        self.rvr = 4000000 / 8 / freq;
        self.tickint = 1;
        self.enable = 1;
    }

};

var ticks_raw: u32 = 0;

fn tick() void {
    ticks_raw += 1;
}

const driver = struct {
    const lld_type = driver_armcortex;

    lld: *driver_armcortex,

    pub fn initialize(self: driver, freq: u32) void {
        interrupt.register(.SysTick, tick);
        self.lld.initialize(freq);
    }

    pub fn count(self: driver) u32 {
        _ = self;
        return @atomicLoad(u32, &ticks_raw, .acquire);
    }

    pub fn delay(self: driver, ticks: u32) void {
        const now = self.count();
        while (self.count() - now < ticks) {}
    }
};

pub const systick = driver { .lld = @ptrFromInt(0xE000E010) };

